require "config"

local spawns = {}

function landBiasNearRespawn(x, y, anywaterprop)

	--the area immediately near the spawn location should have been explored
	--	by the first 15 minutes at worst.
	--so as no new near-spawn areas are getting discovered,
	--the frequent calculation of the distances from the spawn points
	--	is just an unnecessary overhead.
	--so we can speed things up by simply setting the bias to 0
	--	after the initial grace period
	--assuming retrogeneration is disabled
	if game.tick > 15*60*60 and not Config.retrogen then
		return 0
	end

	if #spawns == 0 and game.tick < 60 then
		for i = 1,#game.players do
			spawns[#spawns+1] = game.players[i].position
		end
	end
	
	local maxbiasdist = 150 * Config.landBiasNearRespawnScale
	local minforcedlanddist = 50 * Config.landBiasNearRespawnScale

	--find the distance from the NEAREST player spawn point
	local currentdist = maxbiasdist
	for i = 1,#spawns do
		local spawn = spawns[i]
		local dx = x-spawn.x
		local dy = y-spawn.y
		local dist = math.sqrt(dx*dx+dy*dy)
		if dist < currentdist then
			currentdist = dist
		end
	end

	if currentdist >= maxbiasdist then
		return 0
	else
		--local distancefactor = currentdist / maxbiasdist
		--we want to scale linearly, so return A*distancefactor + B
		--	where minforcedlanddist/maxbiasdist * A + B == anywaterprop
		--	and maxbiasdist/maxbiasdist * A + B == 0
		--    	so A + B = 0
		--		so B = -A
		--		so minforcedlanddist/maxbiasdist * A - A = anywaterprop
		--		so A * (minforcedlanddist/maxbiasdist - 1) = anywaterprop
		--		so A = anywaterprop / (minforcedlanddist/maxbiasdist - 1)
		--		so return (anywaterprop / (minforcedlanddist/maxbiasdist - 1)) * (distancefactor - 1)
		return (anywaterprop / (minforcedlanddist/maxbiasdist - 1)) * (currentdist/maxbiasdist - 1)
	end
end